@@ -4,7 +4,8 @@ use Mojo::Base -base;
 use Mojo::Util qw(html_unescape xml_escape);
 use Scalar::Util 'weaken';
 
-has [qw(tree xml)];
+has tree => sub { ['root'] };
+has 'xml';
 
 my $ATTR_RE = qr/
   ([^<>=\s\/]+|\/)   # Key
@@ -104,7 +105,7 @@ sub parse {
 
   my $xml = $self->xml;
   my $current = my $tree = ['root'];
-  while ($html =~ m/\G$TOKEN_RE/gcso) {
+  while ($html =~ /\G$TOKEN_RE/gcso) {
     my ($text, $doctype, $comment, $cdata, $pi, $tag, $runaway)
       = ($1, $2, $3, $4, $5, $6, $11);
 
@@ -130,7 +131,7 @@ sub parse {
           # Empty tag
           ++$closing and next if $key eq '/';
 
-          $attrs{$key} = defined $value ? html_unescape($value) : $value;
+          $attrs{$key} = defined $value ? html_unescape $value : $value;
         }
 
         # "image" is an alias for "img"
@@ -199,7 +200,7 @@ sub _render {
   my ($tree, $xml) = @_;
 
   # Text (escaped)
-  return '' unless my $type = $tree->[0];
+  my $type = $tree->[0];
   return xml_escape $tree->[1] if $type eq 'text';
 
   # Raw text
