@@ -1,5 +1,5 @@
 package Mojo::IOLoop;
-use Mojo::Base -base;
+use Mojo::Base 'Mojo::EventEmitter';
 
 # "Professor: Amy, technology isn't intrinsically good or evil. It's how it's
 #             used. Like the death ray."
@@ -9,16 +9,14 @@ use Mojo::IOLoop::Delay;
 use Mojo::IOLoop::Server;
 use Mojo::IOLoop::Stream;
 use Mojo::Reactor::Poll;
-use Mojo::Util qw(md5_sum steady_time);
+use Mojo::Util qw(deprecated md5_sum steady_time);
 use Scalar::Util qw(blessed weaken);
 
 use constant DEBUG => $ENV{MOJO_IOLOOP_DEBUG} || 0;
 
-has accept_interval => 0.025;
-has [qw(lock unlock)];
 has max_accepts     => 0;
 has max_connections => 1000;
-has multi_accept    => 50;
+has multi_accept    => sub { shift->max_connections > 50 ? 50 : 1 };
 has reactor         => sub {
   my $class = Mojo::Reactor::Poll->detect;
   warn "-- Reactor initialized ($class)\n" if DEBUG;
@@ -31,6 +29,12 @@ $SIG{PIPE} = 'IGNORE';
 # Initialize singleton reactor early
 __PACKAGE__->singleton->reactor;
 
+# DEPRECATED in Tiger Face!
+sub accept_interval {
+  deprecated 'Mojo::IOLoop::accept_interval is DEPRECATED';
+  @_ > 1 ? shift : undef;
+}
+
 sub acceptor {
   my ($self, $acceptor) = (_instance(shift), @_);
 
@@ -39,11 +43,11 @@ sub acceptor {
 
   # Connect acceptor with reactor
   my $id = $self->_id;
-  $self->{acceptors}{$id} = $acceptor;
+  $self->{acceptors}{$id} = $acceptor->multi_accept($self->multi_accept);
   weaken $acceptor->reactor($self->reactor)->{reactor};
 
   # Allow new acceptor to get picked up
-  $self->_not_accepting;
+  $self->_not_accepting->_maybe_accepting;
 
   return $id;
 }
@@ -51,9 +55,6 @@ sub acceptor {
 sub client {
   my ($self, $cb) = (_instance(shift), pop);
 
-  # Make sure timers are running
-  $self->_recurring;
-
   my $id = $self->_id;
   my $client = $self->{connections}{$id}{client} = Mojo::IOLoop::Client->new;
   weaken $client->reactor($self->reactor)->{reactor};
@@ -86,6 +87,12 @@ sub delay {
 
 sub is_running { _instance(shift)->reactor->is_running }
 
+# DEPRECATED in Tiger Face!
+sub lock {
+  deprecated 'Mojo::IOLoop::lock is DEPRECATED';
+  @_ > 1 ? shift : undef;
+}
+
 sub next_tick {
   my ($self, $cb) = (_instance(shift), @_);
   weaken $self;
@@ -105,10 +112,9 @@ sub remove {
 
 sub reset {
   my $self = _instance(shift);
-  $self->_remove($_)
-    for keys %{$self->{acceptors}}, keys %{$self->{connections}};
+  delete @$self{qw(accepting acceptors connections stop)};
   $self->reactor->reset;
-  $self->$_ for qw(_stop stop);
+  $self->stop;
 }
 
 sub server {
@@ -119,17 +125,17 @@ sub server {
   $server->on(
     accept => sub {
 
-      # Release accept mutex
-      $self->_not_accepting;
-
       # Enforce connection limit (randomize to improve load balancing)
       if (my $max = $self->max_accepts) {
         $self->{accepts} //= $max - int rand $max / 2;
-        $self->max_connections(0) if ($self->{accepts} -= 1) <= 0;
+        $self->stop_gracefully if ($self->{accepts} -= 1) <= 0;
       }
 
       my $stream = Mojo::IOLoop::Stream->new(pop);
       $self->$cb($stream, $self->stream($stream));
+
+      # Stop accepting if connection limit has been reached
+      $self->_not_accepting if $self->_limit;
     }
   );
   $server->listen(@_);
@@ -147,6 +153,11 @@ sub start {
 
 sub stop { _instance(shift)->reactor->stop }
 
+sub stop_gracefully {
+  my $self = _instance(shift)->_not_accepting;
+  $self->{stop} ||= $self->emit('finish')->recurring(1 => \&_stop);
+}
+
 sub stream {
   my ($self, $stream) = (_instance(shift), @_);
   return ($self->{connections}{$stream} || {})->{stream} unless ref $stream;
@@ -155,26 +166,10 @@ sub stream {
 
 sub timer { shift->_timer(timer => @_) }
 
-sub _accepting {
-  my $self = shift;
-
-  # Check if we have acceptors
-  my $acceptors = $self->{acceptors} ||= {};
-  return $self->_remove(delete $self->{accept}) unless keys %$acceptors;
-
-  # Check connection limit
-  my $i   = keys %{$self->{connections}};
-  my $max = $self->max_connections;
-  return unless $i < $max;
-
-  # Acquire accept mutex
-  if (my $cb = $self->lock) { return unless $cb->(!$i) }
-  $self->_remove(delete $self->{accept});
-
-  # Check if multi-accept is desirable
-  my $multi = $self->multi_accept;
-  $_->multi_accept($max < $multi ? 1 : $multi)->start for values %$acceptors;
-  $self->{accepting} = 1;
+# DEPRECATED in Tiger Face!
+sub unlock {
+  deprecated 'Mojo::IOLoop::unlock is DEPRECATED';
+  @_ > 1 ? shift : undef;
 }
 
 sub _id {
@@ -187,24 +182,24 @@ sub _id {
 
 sub _instance { ref $_[0] ? $_[0] : $_[0]->singleton }
 
-sub _not_accepting {
+sub _limit {
   my $self = shift;
-
-  # Make sure timers are running
-  $self->_recurring;
-
-  # Release accept mutex
-  return unless delete $self->{accepting};
-  return unless my $cb = $self->unlock;
-  $cb->();
-
-  $_->stop for values %{$self->{acceptors} || {}};
+  return 1 if $self->{stop};
+  return keys %{$self->{connections}} >= $self->max_connections;
 }
 
-sub _recurring {
+sub _maybe_accepting {
   my $self = shift;
-  $self->{accept} ||= $self->recurring($self->accept_interval => \&_accepting);
-  $self->{stop} ||= $self->recurring(1 => \&_stop);
+  return if $self->{accepting} || $self->_limit;
+  $_->start for values %{$self->{acceptors} || {}};
+  $self->{accepting} = 1;
+}
+
+sub _not_accepting {
+  my $self = shift;
+  return $self unless delete $self->{accepting};
+  $_->stop for values %{$self->{acceptors} || {}};
+  return $self;
 }
 
 sub _remove {
@@ -215,28 +210,28 @@ sub _remove {
   return if $reactor->remove($id);
 
   # Acceptor
-  if (delete $self->{acceptors}{$id}) { $self->_not_accepting }
+  return $self->_not_accepting->_maybe_accepting
+    if delete $self->{acceptors}{$id};
 
   # Connection
-  else { delete $self->{connections}{$id} }
+  return unless delete $self->{connections}{$id};
+  $self->_maybe_accepting;
+  warn "-- $id <<< $$ (@{[scalar keys %{$self->{connections}}]})\n" if DEBUG;
 }
 
 sub _stop {
   my $self = shift;
-  return      if keys %{$self->{connections}};
-  $self->stop if $self->max_connections == 0;
-  return      if keys %{$self->{acceptors}};
-  $self->{$_} && $self->_remove(delete $self->{$_}) for qw(accept stop);
+  return if keys %{$self->{connections}};
+  $self->_remove(delete $self->{stop});
+  $self->stop;
 }
 
 sub _stream {
   my ($self, $stream, $id) = @_;
 
-  # Make sure timers are running
-  $self->_recurring;
-
   # Connect stream with reactor
   $self->{connections}{$id}{stream} = $stream;
+  warn "-- $id >>> $$ (@{[scalar keys %{$self->{connections}}]})\n" if DEBUG;
   weaken $stream->reactor($self->reactor)->{reactor};
   weaken $self;
   $stream->on(close => sub { $self && $self->_remove($id) });
@@ -334,35 +329,25 @@ C<MOJO_NO_TLS> environment variables.
 
 See L<Mojolicious::Guides::Cookbook/"REAL-TIME WEB"> for more.
 
+=head1 EVENTS
+
+L<Mojo::IOLoop> inherits all events from L<Mojo::EventEmitter> and can emit
+the following new ones.
+
+=head2 finish
+
+  $loop->on(finish => sub {
+    my $loop = shift;
+    ...
+  });
+
+Emitted when the event loop wants to shut down gracefully and is just waiting
+for all existing connections to be closed.
+
 =head1 ATTRIBUTES
 
 L<Mojo::IOLoop> implements the following attributes.
 
-=head2 accept_interval
-
-  my $interval = $loop->accept_interval;
-  $loop        = $loop->accept_interval(0.5);
-
-Interval in seconds for trying to reacquire the accept mutex, defaults to
-C<0.025>. Note that changing this value can affect performance and idle CPU
-usage.
-
-=head2 lock
-
-  my $cb = $loop->lock;
-  $loop  = $loop->lock(sub {...});
-
-A callback for acquiring the accept mutex, used to sync multiple server
-processes. The callback should return true or false. Note that exceptions in
-this callback are not captured.
-
-  $loop->lock(sub {
-    my $blocking = shift;
-
-    # Got the accept mutex, start accepting new connections
-    return 1;
-  });
-
 =head2 max_accepts
 
   my $max = $loop->max_accepts;
@@ -381,16 +366,15 @@ randomly to improve load balancing between multiple server processes.
 
 The maximum number of concurrent connections this event loop is allowed to
 handle before stopping to accept new incoming connections, defaults to
-C<1000>. Setting the value to C<0> will make this event loop stop accepting
-new connections and allow it to shut down gracefully without interrupting
-existing connections.
+C<1000>.
 
 =head2 multi_accept
 
   my $multi = $loop->multi_accept;
   $loop     = $loop->multi_accept(100);
 
-Number of connections to accept at once, defaults to C<50>.
+Number of connections to accept at once, defaults to C<50> or C<1>, depending
+on if the value of L</"max_connections"> is smaller than C<50>.
 
 =head2 reactor
 
@@ -413,18 +397,10 @@ L<Mojo::Reactor/"error">.
   # Remove handle again
   $loop->reactor->remove($handle);
 
-=head2 unlock
-
-  my $cb = $loop->unlock;
-  $loop  = $loop->unlock(sub {...});
-
-A callback for releasing the accept mutex, used to sync multiple server
-processes. Note that exceptions in this callback are not captured.
-
 =head1 METHODS
 
-L<Mojo::IOLoop> inherits all methods from L<Mojo::Base> and implements the
-following new ones.
+L<Mojo::IOLoop> inherits all methods from L<Mojo::EventEmitter> and implements
+the following new ones.
 
 =head2 acceptor
 
@@ -613,6 +589,13 @@ loop object from everywhere inside the process.
   my $id = Mojo::IOLoop->timer(3 => sub { say 'Timeout!' });
   Mojo::IOLoop->singleton->reactor->again($id);
 
+  # Turn file descriptor into handle and watch if it becomes readable
+  my $handle = IO::Handle->new_from_fd($fd, 'r');
+  Mojo::IOLoop->singleton->reactor->io($handle => sub {
+    my ($reactor, $writable) = @_;
+    say $writable ? 'Handle is writable' : 'Handle is readable';
+  })->watch($handle, 1, 0);
+
 =head2 start
 
   Mojo::IOLoop->start;
@@ -632,6 +615,14 @@ some reactors stop automatically if there are no events being watched anymore.
 Stop the event loop, this will not interrupt any existing connections and the
 event loop can be restarted by running L</"start"> again.
 
+=head2 stop_gracefully
+
+  Mojo::IOLoop->stop_gracefully;
+  $loop->stop_gracefully;
+
+Stop accepting new connections and wait for all existing connections to be
+closed before stopping the event loop.
+
 =head2 stream
 
   my $stream = Mojo::IOLoop->stream($id);
