@@ -188,7 +188,7 @@ sub start_line_size { length shift->build_start_line }
 sub text {
   my $self    = shift;
   my $body    = $self->body;
-  my $charset = $self->content->charset;
+  my $charset = $self->content->charset || $self->default_charset;
   return $charset ? decode($charset, $body) // $body : $body;
 }
 
@@ -219,8 +219,7 @@ sub uploads {
 sub _build {
   my ($self, $method) = @_;
 
-  my $buffer = '';
-  my $offset = 0;
+  my ($buffer, $offset) = ('', 0);
   while (1) {
 
     # No chunk yet, try again
@@ -373,7 +372,9 @@ Message content, defaults to a L<Mojo::Content::Single> object.
   my $charset = $msg->default_charset;
   $msg        = $msg->default_charset('UTF-8');
 
-Default charset used for form-data parsing, defaults to C<UTF-8>.
+Default charset used by L</"text"> and to extract data from
+C<application/x-www-form-urlencoded> or C<multipart/form-data> message body,
+defaults to C<UTF-8>.
 
 =head2 max_line_size
 
@@ -478,12 +479,13 @@ Access message cookies. Meant to be overloaded in a subclass.
   my $dom        = $msg->dom;
   my $collection = $msg->dom('a[href]');
 
-Turns message body into a L<Mojo::DOM> object and takes an optional selector
-to call the method L<Mojo::DOM/"find"> on it right away, which returns a
-L<Mojo::Collection> object. Note that this method caches all data, so it
-should not be called before the entire message body has been received. The
-whole message body needs to be loaded into memory to parse it, so you have to
-make sure it is not excessively large, there's a 16MB limit by default.
+Retrieve message body from L</"text"> and turn it into a L<Mojo::DOM> object,
+an optional selector can be used to call the method L<Mojo::DOM/"find"> on it
+right away, which then returns a L<Mojo::Collection> object. Note that this
+method caches all data, so it should not be called before the entire message
+body has been received. The whole message body needs to be loaded into memory
+to parse it, so you have to make sure it is not excessively large, there's a
+16MB limit by default.
 
   # Perform "find" right away
   say $msg->dom('h1, h2, h3')->map('text')->join("\n");
@@ -575,6 +577,9 @@ Size of headers in bytes.
 
 Message headers, usually a L<Mojo::Headers> object.
 
+  # Longer version
+  my $headers = $msg->content->headers;
+
 =head2 is_finished
 
   my $bool = $msg->is_finished;
@@ -621,8 +626,8 @@ Size of the start-line in bytes.
 
   my $str = $msg->text;
 
-Retrieve L</"body"> and try to decode it if a charset could be extracted with
-L<Mojo::Content/"charset">.
+Retrieve L</"body"> and try to decode it with L<Mojo::Content/"charset"> or
+L</"default_charset">.
 
 =head2 to_string
 
