@@ -16,16 +16,22 @@ has listen => sub { [split ',', $ENV{MOJO_LISTEN} || 'http://*:3000'] };
 has max_requests => 25;
 
 sub DESTROY {
+  return if Mojo::Util::_global_destruction();
   my $self = shift;
-  return unless my $loop = $self->ioloop;
   $self->_remove($_) for keys %{$self->{connections} || {}};
+  my $loop = $self->ioloop;
   $loop->remove($_) for @{$self->acceptors};
 }
 
 sub run {
   my $self = shift;
-  local $SIG{INT} = local $SIG{TERM} = sub { $self->ioloop->stop };
+
+  # Make sure the event loop can be stopped in regular intervals
+  my $loop = $self->ioloop;
+  my $int = $loop->recurring(1 => sub { });
+  local $SIG{INT} = local $SIG{TERM} = sub { $loop->stop };
   $self->start->setuidgid->ioloop->start;
+  $loop->remove($int);
 }
 
 sub start {
@@ -33,6 +39,7 @@ sub start {
 
   # Resume accepting connections
   my $loop = $self->ioloop;
+  if (my $max = $self->max_clients) { $loop->max_connections($max) }
   if (my $servers = $self->{servers}) {
     push @{$self->acceptors}, $loop->acceptor(delete $servers->{$_})
       for keys %$servers;
@@ -40,7 +47,6 @@ sub start {
 
   # Start listening
   else { $self->_listen($_) for @{$self->listen} }
-  if (my $max = $self->max_clients) { $loop->max_connections($max) }
 
   return $self;
 }
@@ -100,16 +106,17 @@ sub _close {
 }
 
 sub _finish {
-  my ($self, $id, $tx) = @_;
+  my ($self, $id) = @_;
 
   # Always remove connection for WebSockets
+  my $c = $self->{connections}{$id};
+  return unless my $tx = $c->{tx};
   return $self->_remove($id) if $tx->is_websocket;
 
   # Finish transaction
   $tx->server_close;
 
   # Upgrade connection to WebSocket
-  my $c = $self->{connections}{$id};
   if (my $ws = $c->{tx} = delete $c->{ws}) {
 
     # Successful upgrade
@@ -191,8 +198,8 @@ sub _read {
     if (($c->{requests} || 0) >= $self->max_requests) || $tx->req->error;
 
   # Finish or start writing
-  if ($tx->is_finished) { $self->_finish($id, $tx) }
-  elsif ($tx->is_writing) { $self->_write($id) }
+  if    ($tx->is_finished) { $self->_finish($id) }
+  elsif ($tx->is_writing)  { $self->_write($id) }
 }
 
 sub _remove {
@@ -220,10 +227,10 @@ sub _write {
   my $cb = sub { $self->_write($id) };
   if ($tx->is_finished) {
     if ($tx->has_subscribers('finish')) {
-      $cb = sub { $self->_finish($id, $tx) }
+      $cb = sub { $self->_finish($id) }
     }
     else {
-      $self->_finish($id, $tx);
+      $self->_finish($id);
       return unless $c->{tx};
     }
   }
@@ -276,6 +283,15 @@ C<MOJO_NO_TLS> environment variables.
 
 See L<Mojolicious::Guides::Cookbook/"DEPLOYMENT"> for more.
 
+=head1 SIGNALS
+
+The L<Mojo::Server::Daemon> process can be controlled at runtime with the
+following signals.
+
+=head2 INT, TERM
+
+Shut down server immediately.
+
 =head1 EVENTS
 
 L<Mojo::Server::Daemon> inherits all events from L<Mojo::Server>.
